our $txt = <<HEREDOC;
Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nulla tincidunt fringilla lectus, et vestibulum sapien consectetur nec. Mauris at auctor ipsum. Phasellus a massa non lorem vulputate lobortis. Sed rutrum, eros vitae dignissim tempor, dui odio vulputate quam, eu suscipit turpis lorem commodo risus. Nulla augue metus, dictum in hendrerit et, ullamcorper vel tortor. Praesent at risus felis. Donec iaculis felis sit amet turpis laoreet volutpat. Nam eros tortor, fermentum id tristique in, hendrerit a nulla. Aliquam tincidunt ante ac elit ultrices et hendrerit ligula rhoncus. Cum sociis natoque penatibus et magnis dis parturient montes, nascetur ridiculus mus. Aenean vehicula sollicitudin nisi, sed commodo ipsum bibendum et. Proin vehicula aliquam placerat. Nulla congue sem nec urna venenatis semper sed sed tortor. Donec ut lectus nec risus mattis egestas eget sit amet lorem. In eros lectus, porta sed placerat at, aliquam a nunc.

Nam commodo blandit aliquet. Suspendisse a orci arcu, sed euismod nibh. Curabitur at placerat est. Maecenas rutrum lacinia leo, sit amet cursus ante porttitor eu. Morbi a nisi in justo fringilla accumsan vehicula vel ante. Aliquam porta tempus ullamcorper. Etiam ac velit justo, vitae venenatis sem. Curabitur sit amet nibh magna. Proin adipiscing, dui et varius ultricies, purus sem luctus libero, id elementum neque augue et eros. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae; Curabitur rutrum congue arcu vel ultricies. Mauris consectetur sodales lorem et volutpat.

Maecenas dictum condimentum erat vel consequat. Quisque eu risus libero, non ornare quam. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut viverra nisl arcu, nec mollis quam. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Quisque malesuada vestibulum est, vel lacinia lorem euismod nec. Nulla massa odio, consectetur ac scelerisque at, fermentum in metus. Duis vitae ipsum lacus, id rutrum felis. Nulla dapibus est eget mauris viverra malesuada sit amet sit amet elit. Sed sodales, turpis vitae eleifend dapibus, odio magna iaculis ante, quis sagittis tellus dolor nec risus. Fusce posuere enim vel dolor lobortis rutrum. Duis in erat nunc, et egestas libero. Morbi pellentesque semper eros ac vulputate. Donec ultricies tincidunt augue.
HEREDOC
